﻿using Microsoft.AspNetCore.Identity.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore;
using MTWorkHR.Core.Entities;
using MTWorkHR.Core.Entities.Base;
using MTWorkHR.Core.Global;
using MTWorkHR.Infrastructure.Entities;
using MTWorkHR.Infrastructure.Data;
using System.Reflection.Emit;


namespace MTWorkHR.Infrastructure.DBContext
{
    public class HRDataContext : IdentityDbContext<ApplicationUser, ApplicationRole, string>
    {
        private readonly GlobalInfo _globalInfo;

        public HRDataContext(DbContextOptions<HRDataContext> options, GlobalInfo globalInfo) : base(options) {
            this._globalInfo = globalInfo;

        }
        public DbSet<AttachmentType> AttachmentTypes { get; set; }
        public DbSet<UserAttachment> UserAttachments { get; set; }
        public DbSet<Permission> Permissions { get; set; }
        public DbSet<RolePermission> RolePermissions { get; set; }



        //-------------------------------------
        public DbSet<Company> Companies { get; set; }
        public DbSet<Project> Projects { get; set; }
        //        public DbSet<AttachmentType> AttachmentTypes { get; set; }
        public DbSet<UserTaskStatus> UserTaskStatuses { get; set; }
        public DbSet<UserTask> UserTasks { get; set; }
        public DbSet<UserTaskAttachment> UserTaskAttachments { get; set; }
        public DbSet<UserTaskHistory> UserTaskHistories { get; set; }
        public DbSet<Team> Teams { get; set; }
        public DbSet<Meeting> Meetings { get; set; }
        //------------------------Logs------------------------
        public DbSet<UserLog> UserLogs { get; set; }
        public DbSet<AuthLog> AuthLogs { get; set; }
        public DbSet<FileLog> FileLogs { get; set; }
        public DbSet<RoleLog> RoleLogs { get; set; }
        public DbSet<SettingLog> SettingLogs { get; set; }
        public DbSet<UserTaskLog> UserTaskLogs { get; set; }
        public DbSet<TeamLog> TeamLogs { get; set; }
        public DbSet<MeetingLog> MeetingLogs { get; set; }
        //----------------------------------------
        protected override void OnModelCreating(ModelBuilder builder)
        {
            base.OnModelCreating(builder);
            SetGlobalFilters(builder);
            builder.ApplyConfigurationsFromAssembly(typeof(HRDataContext).Assembly);
        }

        private void SetGlobalFilters(ModelBuilder builder)
        {
            builder.SetQueryFilterOnAllEntities<ISoftDelete>(p => !p.IsDeleted);
        }
        #region SaveChanges 
        public override Task<int> SaveChangesAsync(CancellationToken cancellationToken = default)
        {
            BeforeSaveProccess();

            return base.SaveChangesAsync(cancellationToken);
        }

        public override int SaveChanges()
        {
            BeforeSaveProccess();

            return base.SaveChanges();
        }

        public override int SaveChanges(bool acceptAllChangesOnSuccess)
        {
            BeforeSaveProccess();

            return base.SaveChanges(acceptAllChangesOnSuccess);
        }

        private void BeforeSaveProccess()
        {
            var changes = from e in this.ChangeTracker.Entries()
                          where e.State != EntityState.Unchanged
                          select e;

            foreach (var change in changes)
            {
                if (change.State == EntityState.Added)
                {
                    if (change.Entity is IAudit)
                    {
                        ((IAudit)change.Entity).CreateUser = _globalInfo.UserId;
                        ((IAudit)change.Entity).CreateDate = DateTime.Now;
                    }
                }
                else if (change.State == EntityState.Modified)
                {

                    if (change.Entity is IAudit
                        && ((change.Entity is IFullAudit && !((IFullAudit)change.Entity).IsDeleted) || change.Entity is not IFullAudit))
                    {
                        ((IAudit)change.Entity).UpdateUser = _globalInfo.UserId;
                        ((IAudit)change.Entity).UpdateDate = DateTime.Now;
                    }

                    if (change.Entity is IFullAudit && ((IFullAudit)change.Entity).IsDeleted)
                    {
                        ((IFullAudit)change.Entity).DeleteUserId = _globalInfo.UserId;
                    }

                }
                else if (change.State == EntityState.Deleted)
                {

                }
            }
        }
        #endregion       
    }
}
